/*
 * linux/arch/mips/vr41xx/giuinfo.c
 *
 * Shows General Purpose I/O status in /proc/giuinfo
 *
 * Copyright (C) 1999 Bradley D. LaRonde
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 */

#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/vr41xx.h>

int get_giuinfo(char *buffer, char **buffer_location, off_t offset,
	int buffer_length, int zero)
{
	// return everything at once
	if ( offset > 0)
		return 0;

	// return the giu register info
	return sprintf(buffer,
		"giuiosell:  0x%04hx\n"
		"giuioselh:  0x%04hx\n"
		"giupiodl:   0x%04hx\n"
		"giupiodh:   0x%04hx\n"
		"giupodatl:  0x%04hx\n",
		*VR41XX_GIUIOSELL, *VR41XX_GIUIOSELH, *VR41XX_GIUPIODL,
		*VR41XX_GIUPIODH, *VR41XX_GIUPODATL);
}

struct proc_dir_entry giuinfo_entry = {
	0,                   // dynamic inode assigned by proc_register
	7, "giuinfo",        // name length and name
	S_IFREG | S_IRUGO,   // mode
	1,                   // number of links
	0, 0,                // uid, gid
	0,                   // size for ls
	NULL,                // inode ops
	get_giuinfo,         // get_info
	NULL                 // fill_inode
};

void __init giuinfo_init(void)
{
	proc_register(&proc_root, &giuinfo_entry);
}

