/*
 * linux/arch/mips/vr41xx/kbd.c
 *
 * Vr41xx keyboard (KIU) driver
 *
 * Copyright (C) 1999 Bradley D. LaRonde
 * 
 * Oct 1999 Hiroshi Kawashima (kawashima@iname.com) adjusted scan lines for MobilPro.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 */

#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/mm.h>
#include <linux/kbd_kern.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <asm/keyboard.h>
#include <linux/kbd_ll.h>
#include <asm/vr41xx.h>

void kbd_leds(unsigned char leds)
{
	return;
}

int kbd_setkeycode(unsigned int scancode, unsigned int keycode)
{
	return -EINVAL;
}

int kbd_getkeycode(unsigned int scancode)
{
	return -EINVAL;
}

int kbd_translate(unsigned char scancode, unsigned char *keycode,
	char raw_mode)
{
	*keycode = scancode;
	return 1;
}

char kbd_unexpected_up(unsigned char keycode)
{
	printk("vr41xx_kiu: unexpected up, keycode 0x%x\n", keycode);
	return 0x80;
}

inline static void handle_cmp_kiudat_bit(unsigned short kiudat, unsigned short cmp_kiudat, int scancode_base,
	unsigned short bitmask, int bit)
{
	// did this bit change?
	if ( cmp_kiudat & bitmask ) {
		// send the scancode and down or up
		handle_scancode(scancode_base + bit, kiudat & bitmask ? 1 : 0);
	}
}

inline static void handle_kiudat(unsigned short kiudat, unsigned short last_kiudat, int scancode_base)
{
	unsigned short cmp_kiudat;

	// compare this data with the previous data, setting only bits that changed
	cmp_kiudat = kiudat ^ last_kiudat;

	// nothing changed - most common case
	if ( cmp_kiudat == 0 )
		return;

	// handle each bit
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0001, 0);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0002, 1);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0004, 2);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0008, 3);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0010, 4);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0020, 5);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0040, 6);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0080, 7);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0100, 8);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0200, 9);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0400, 10);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x0800, 11);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x1000, 12);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x2000, 13);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x4000, 14);
	handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode_base, 0x8000, 15);
}

#define KIUDAT_REGISTERS 6

inline static void handle_kiu_data_ready(void)
{
	static unsigned short last_kiudat[KIUDAT_REGISTERS] = {0,0,0,0,0,0};
	unsigned short kiudat[KIUDAT_REGISTERS];
	
	// read the scan data
	kiudat[0] = *VR41XX_KIUDAT0;
	kiudat[1] = *VR41XX_KIUDAT1;
	kiudat[2] = *VR41XX_KIUDAT2;
	kiudat[3] = *VR41XX_KIUDAT3;
	kiudat[4] = *VR41XX_KIUDAT4;
	kiudat[5] = *VR41XX_KIUDAT5;

	// handle each register based on it's previous value
	handle_kiudat(kiudat[0], last_kiudat[0], 0);
	handle_kiudat(kiudat[1], last_kiudat[1], 16);
	handle_kiudat(kiudat[2], last_kiudat[2], 32);
	handle_kiudat(kiudat[3], last_kiudat[3], 48);
	handle_kiudat(kiudat[4], last_kiudat[4], 64);
	handle_kiudat(kiudat[5], last_kiudat[5], 80);

	// save the current scan data for the next time
	last_kiudat[0] = kiudat[0];
	last_kiudat[1] = kiudat[1];
	last_kiudat[2] = kiudat[2];
	last_kiudat[3] = kiudat[3];
	last_kiudat[4] = kiudat[4];
	last_kiudat[5] = kiudat[5];
}

void kbd_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	short kiuint;

	// get keyboard interrupt status
	kiuint = *VR41XX_KIUINT;

	// clear all keyboard interrupts
	*VR41XX_KIUINT = 0x0007;

	// keyboard data ready?
	if ( kiuint & VR41XX_KDATRDY ) {
		handle_kiu_data_ready();
	}

	// lost keyboard data?
	if ( kiuint & VR41XX_KDATLOST ) {
		printk("vr41xx_kiu: keyboard data lost\n");
	}

	#if 0
	// keyboard input detected?
	if ( kiuint & VR41XX_SCANINT ) {
		printk("vr41xx_kiu: scanint\n");
	}
	#endif
}

void __init kbd_init_hw(void)
{
	printk("Keyboard driver version 0.1 for VR41XX KIU\n");

	// turn on the KIU clock
	*VR41XX_CMUCLKMSK |= 0x0008;

	// reset the KIU
	*VR41XX_KIURST = 0x0001;

	barrier();

#ifdef CONFIG_VADEM_CLIO
	// use only 8 scan lines
	*VR41XX_SCANLINE = 0x0002;
#else
	// use all 12 scan lines
	// needed for NEC MobileGear-II (tested on R300) and probbably other MobilePro/Gears
	*VR41XX_SCANLINE = 0x0000;
#endif

	// enable scan, auto scan, auto stop, and one stop rep
	*VR41XX_KIUSCANREP = VR41XX_KEYEN | VR41XX_ATSCAN | VR41XX_ATSTP | (1 << 4);

	// set stabilization, off, and wait times in 60us increments
	// I just picked a short (but not the shortest) setting for each
	*VR41XX_KIUWKS = 8 | (8 << 5) | (8 << 10);

	// set time between scanner cycles in 30us increments
	// I picked 10000us - 1/100 second between scans
	// setting this really short resulted in lost keyboard data
	*VR41XX_KIUWKI = 333;

	barrier();

	// clear any pending interrupts
	*VR41XX_KIUINT = 0x0007; 

	barrier();

	// install the interrupt handler
	// should this be SA_SHIRQ?
	request_irq(VR41XX_IRQ_KIU, kbd_irq_handler, 0, "keyboard", NULL);

	// unmask the keyboard interrupts that we are interested in
	*VR41XX_MKIUINTREG = 0x7;
}

