/*
 * linux/arch/mips/vr41xx/rtc.c
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1999 Bradley D. LaRonde
 *
 */

#include <linux/mc146818rtc.h>

static int get_elapsed_time_seconds(void)
{
	// elapsed time counts up 32.768 kHz
	short etimem = *((short*)0xab0000c2);
	short etimel = *((short*)0xab0000c0);
	return (etimem << 1) + (etimel >> 14);
}

static unsigned char rtc_read_data(unsigned long addr)
{
	switch (addr) {
		case RTC_SECONDS:
			return get_elapsed_time_seconds() % 60;
		case RTC_MINUTES:
			return (get_elapsed_time_seconds() / 60) % 60;
		case RTC_HOURS:
			return (get_elapsed_time_seconds() / 3600) % 60;

		case RTC_DAY_OF_WEEK:
			return 6;
		case RTC_DAY_OF_MONTH:
			return 23;
		case RTC_MONTH:
			return 7;
		case RTC_YEAR:
			return 99;

		case RTC_SECONDS_ALARM:
			return 0;
		case RTC_MINUTES_ALARM:
			return 0;
		case RTC_HOURS_ALARM:
			return 0;

		default:
			return 0;
	}
}

static void rtc_write_data(unsigned char data, unsigned long addr)
{
	// panic("vr41xx: rtc_write_data not implmented\n");
}

static int rtc_bcd_mode(void)
{
	return 0;
}

struct rtc_ops vr41xx_rtc_ops =
{
    &rtc_read_data,
    &rtc_write_data,
    &rtc_bcd_mode
};

