/*
 * linux/arch/mips/vr41xx/setup.c
 *
 * Setup Vr41xx interrupts
 *
 * Copyright (C) 1999 Bradley D. LaRonde and Michael Klar
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 */

#include <linux/config.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/mc146818rtc.h>
#include <linux/param.h>
#include <linux/console.h>
#include <linux/ide.h>
#include <linux/init.h>
#include <asm/mipsregs.h>
#include <asm/irq.h>
#include <asm/reboot.h>
#include <asm/bootinfo.h>
#include <asm/vr41xx.h>
#ifdef CONFIG_REMOTE_DEBUG
#include <asm/gdb-stub.h>
void breakpoint(void);
#endif

// Unfortunately, the Vrc4171 companion chip requires port numbers to be offset
// 0x01000000 in system bus (ISA-like) IO space.  This isn't a very clean thing
// to do, but since probably nothing else will be sitting on IO space, we just
// handle the difference here:
// Note: This should really be a CONFIG option, but for now, I'm just determining
// it based on device type configured.  This should be any device with Vrc4171
// PCMCIA (and LCD) controller.
// Note2: Not sure if Casio E-10 and E-15 belong in this list.
#if defined(CONFIG_CASIO_E10) \
 || defined(CONFIG_CASIO_E15) \
 || defined(CONFIG_EVEREX_FREESTYLE)
#define ADJUSTED_PORT_BASE (VR41XX_PORT_BASE + 0x01000000)
#else
#define ADJUSTED_PORT_BASE VR41XX_PORT_BASE
#endif

extern struct ide_ops vr41xx_ide_ops;
extern struct rtc_ops vr41xx_rtc_ops;
extern int setup_vr41xx_irq(int irq, struct irqaction *new);
extern asmlinkage void vr41xx_handle_irq(void);
void (*board_time_init)(struct irqaction * irq);
int add_irq_action(int irq, struct irqaction *new);

static struct irqaction cascade = { NULL, SA_INTERRUPT, 0, "cascade", NULL, NULL };

void __init vr41xx_init_irqs(void)
{
 	// Default all ICU IRQs to off
 	*VR41XX_MSYSINT1REG = 0x0100;	// Leave GIU int on in ICU level 1
 	*VR41XX_MSYSINT2REG = 0;
 
 	// We initialize the level 2 ICU registers to all bits disabled.
 	// After this, these registers are the resposibility of whatever
 	// driver requests the IRQ of the corresponding level 1 ICU bit.
 	*VR41XX_MPIUINTREG = 0;
 	*VR41XX_MAIUINTREG = 0;
 	*VR41XX_MKIUINTREG = 0;
 	*VR41XX_MGIUINTLREG = 0;
 	*VR41XX_MDSIUINTREG = 0;
 	*VR41XX_MGIUINTHREG = 0;
 	*VR41XX_MFIRINTREG = 0;
 
 	// This doesn't really add a handler, IRQs 2 or 16 are never reported by the int
 	// vector handler.  What this does is to register the interrupt as non-sharable
 	add_irq_action(VR41XX_IRQ_INT0, &cascade);
 	add_irq_action(VR41XX_IRQ_GIU, &cascade);
}

static void __init vr41xx_irq_setup(void)
{
	set_except_vector(0, vr41xx_handle_irq);
 	vr41xx_init_irqs();

#ifdef CONFIG_REMOTE_DEBUG
 	printk("Setting debug traps - please connect the remote debugger.\n");

	set_debug_traps();

	// you may move this line to whereever you want
	breakpoint();
#endif
}

void __init init_IRQ(void)
{
	// called by init/main.c
	// indirectly calls vr41xx_irq_setup, set below in vr41xx_setup
	irq_setup();
}

void __init board_time_init_not_here(struct irqaction* paction)
{
	// just in case someone tries to call board_time_init
	panic("board_time_init is not here!");
}

void vr41xx_restart(char* c)
{
	void* reset_addr = (void*)0xbfc00000;
	goto *reset_addr;
}

void vr41xx_hibernate(void)
{
	// gas doesn't like the "hibernate" instruction, so hand-code it
	asm(
		".set\tnoreorder\n\t"
		".int\t0x42000023\n\t"
		"nop\n\t"
		"nop\n\t"
		"nop\n\t"
		"nop\n\t"
		".set\treorder\n\t"
	);
}

void __init vr41xx_setup(void)
{
	irq_setup = vr41xx_irq_setup;
	board_time_init = board_time_init_not_here;
	mips_io_port_base = ADJUSTED_PORT_BASE;

#ifdef CONFIG_BLK_DEV_IDE
	ide_ops = &vr41xx_ide_ops;
#endif
	rtc_ops = &vr41xx_rtc_ops;

#ifdef CONFIG_FB
	conswitchp = &dummy_con;
#endif

	_machine_restart = vr41xx_restart;
	_machine_halt = vr41xx_hibernate;
	_machine_power_off = vr41xx_hibernate;
}

void __init kbd_reset_setup(char *str, int *ints)
{
	// process kbd-reset boot parameter
	// in our case, we always reset the keyboard hardware, so this doesn't do anything
}

