/* $Id: vr41xx.h,v 1.9 1999/10/28 19:24:07 brad Exp $
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1999 by Michael Klar
 */
#ifndef __ASM_MIPS_VR41XX_H 
#define __ASM_MIPS_VR41XX_H 

#include <asm/addrspace.h>

// CPU interrupts
#define VR41XX_IRQ_SW1       0  // IP0 - Software interrupt
#define VR41XX_IRQ_SW2       1  // IP1 - Software interrupt
#define VR41XX_IRQ_INT0      2  // IP2 - All but battery, high speed modem, and real time clock
#define VR41XX_IRQ_INT1      3  // IP3 - RTC Long1 (system timer)
#define VR41XX_IRQ_INT2      4  // IP4 - RTC Long2
#define VR41XX_IRQ_INT3      5  // IP5 - High Speed Modem
#define VR41XX_IRQ_INT4      6  // IP6 - Unused
#define VR41XX_IRQ_TIMER     7  // IP7 - Timer interrupt from CPO_COMPARE (Note: RTC Long1 is the system timer.)

// Cascaded from VR41XX_IRQ_INT0 (ICU mapped interrupts)
#define VR41XX_IRQ_BATTERY   8
#define VR41XX_IRQ_POWER     9
#define VR41XX_IRQ_RTCL1     10  // Use VR41XX_IRQ_INT1 instead.
#define VR41XX_IRQ_ETIMER    11
#define VR41XX_IRQ_RFU12     12
#define VR41XX_IRQ_PIU       13
#define VR41XX_IRQ_AIU       14
#define VR41XX_IRQ_KIU       15
#define VR41XX_IRQ_GIU       16  // This is a cascade to IRQs 40-71. Do not use.
#define VR41XX_IRQ_SIU       17
#define VR41XX_IRQ_WRBERR    18
#define VR41XX_IRQ_SOFT      19
#define VR41XX_IRQ_RFU20     20
#define VR41XX_IRQ_DOZEPIU   21
#define VR41XX_IRQ_RFU22     22
#define VR41XX_IRQ_RFU23     23
#define VR41XX_IRQ_RTCL2     24  // Use VR41XX_IRQ_INT2 instead.
#define VR41XX_IRQ_LED       25
#define VR41XX_IRQ_HSP       26  // Use VR41XX_IRQ_INT3 instead.
#define VR41XX_IRQ_TCLK      27
#define VR41XX_IRQ_FIR       28
#define VR41XX_IRQ_DSIU      29
#define VR41XX_IRQ_RFU30     30
#define VR41XX_IRQ_RFU31     31
#define VR41XX_IRQ_RFU32     32
#define VR41XX_IRQ_RFU33     33
#define VR41XX_IRQ_RFU34     34
#define VR41XX_IRQ_RFU35     35
#define VR41XX_IRQ_RFU36     36
#define VR41XX_IRQ_RFU37     37
#define VR41XX_IRQ_RFU38     38
#define VR41XX_IRQ_RFU39     39

// Cascaded from VR41XX_IRQ_GIU
#define VR41XX_IRQ_GPIO0     40
#define VR41XX_IRQ_GPIO1     41
#define VR41XX_IRQ_GPIO2     42
#define VR41XX_IRQ_GPIO3     43
#define VR41XX_IRQ_GPIO4     44
#define VR41XX_IRQ_GPIO5     45
#define VR41XX_IRQ_GPIO6     46
#define VR41XX_IRQ_GPIO7     47
#define VR41XX_IRQ_GPIO8     48
#define VR41XX_IRQ_GPIO9     49
#define VR41XX_IRQ_GPIO10    50
#define VR41XX_IRQ_GPIO11    51
#define VR41XX_IRQ_GPIO12    52
#define VR41XX_IRQ_GPIO13    53
#define VR41XX_IRQ_GPIO14    54
#define VR41XX_IRQ_GPIO15    55
#define VR41XX_IRQ_GPIO16    56
#define VR41XX_IRQ_GPIO17    57
#define VR41XX_IRQ_GPIO18    58
#define VR41XX_IRQ_GPIO19    59
#define VR41XX_IRQ_GPIO20    60
#define VR41XX_IRQ_GPIO21    61
#define VR41XX_IRQ_GPIO22    62
#define VR41XX_IRQ_GPIO23    63
#define VR41XX_IRQ_GPIO24    64
#define VR41XX_IRQ_GPIO25    65
#define VR41XX_IRQ_GPIO26    66
#define VR41XX_IRQ_GPIO27    67
#define VR41XX_IRQ_GPIO28    68
#define VR41XX_IRQ_GPIO29    69
#define VR41XX_IRQ_GPIO30    70
#define VR41XX_IRQ_GPIO31    71

// Alternative to above GPIO IRQ defines
#define VR41XX_IRQ_GPIO(pin) ((VR41XX_IRQ_GPIO0) + (pin))

#define VR41XX_IRQ_MAX       71

#ifndef _LANGUAGE_ASSEMBLY
#define preg8	(volatile unsigned char*)
#define preg16	(volatile unsigned short*)
#define preg32	(volatile unsigned int*)
#else
#define preg8
#define preg16
#define preg32
#endif

#define VR41XX_BCUCNTREG1	preg16(KSEG1 + 0x0B000000)	/* BCU Control Register 1 (R/W) */
#define VR41XX_BCUCNTREG2	preg16(KSEG1 + 0x0B000002)	/* BCU Control Register 2 (R/W) */
#define VR41XX_ROMSIZEREG	preg16(KSEG1 + 0x0B000004)	/* ROM Size Register (R/W) */
#define VR41XX_RAMSIZEREG	preg16(KSEG1 + 0x0B000006)	/* DRAM Size Register (R/W) */
#define VR41XX_BCUSPEEDREG	preg16(KSEG1 + 0x0B00000A)	/* BCU Access Cycle Change Register (R/W) */
#define VR41XX_BCUERRSTREG	preg16(KSEG1 + 0x0B00000C)	/* BCU BUS ERROR Status Register (R/W) */
#define VR41XX_BCURFCNTREG	preg16(KSEG1 + 0x0B00000E)	/* BCU Refresh Control Register (R/W) */
#define VR41XX_REVIDREG		preg16(KSEG1 + 0x0B000010)	/* Revision ID Register (R) */
#define VR41XX_BCURFCOUNTREG	preg16(KSEG1 + 0x0B000012)	/* BCU Refresh Count Register (R/W) */
#define VR41XX_CLKSPEEDREG	preg16(KSEG1 + 0x0B000014)	/* Clock Speed Register (R) */
#define VR41XX_BCUCNTREG3	preg16(KSEG1 + 0x0B000016)	/* BCU Control Register 3 (R/W) */
#define VR41XX_SDRAMMODEREG	preg16(KSEG1 + 0x0B00001A)	/* SDRAM Mode Register */
#define VR41XX_SROMMODEREG	preg16(KSEG1 + 0x0B00001C)	/* SROM Mode Register */
#define VR41XX_SDRAMCNTREG	preg16(KSEG1 + 0x0B00001E)	/* SDRAM Control Register */
#define VR41XX_BCUTOUTCNTREG	preg16(KSEG1 + 0x0B000300)	/* BCU Timeout Control Register */
#define VR41XX_BCUTOUTCOUNTREG	preg16(KSEG1 + 0x0B000302)	/* BCU Timeout Count Register */

#define VR41XX_AIUIBALREG	preg16(KSEG1 + 0x0B000020)	/* AIU IN DMA Base Address Register Low (R/W) */
#define VR41XX_AIUIBAHREG	preg16(KSEG1 + 0x0B000022)	/* AIU IN DMA Base Address Register High (R/W) */
#define VR41XX_AIUIALREG	preg16(KSEG1 + 0x0B000024)	/* AIU IN DMA Address Register Low (R/W) */
#define VR41XX_AIUIAHREG	preg16(KSEG1 + 0x0B000026)	/* AIU IN DMA Address Register High (R/W) */
#define VR41XX_AIUOBALREG	preg16(KSEG1 + 0x0B000028)	/* AIU OUT DMA Base Address Register Low (R/W) */
#define VR41XX_AIUOBAHREG	preg16(KSEG1 + 0x0B00002A)	/* AIU OUT DMA Base Address Register High (R/W) */
#define VR41XX_AIUOALREG	preg16(KSEG1 + 0x0B00002C)	/* AIU OUT DMA Address Register Low (R/W) */
#define VR41XX_AIUOAHREG	preg16(KSEG1 + 0x0B00002E)	/* AIU OUT DMA Address Register High (R/W) */
#define VR41XX_FIRBALREG	preg16(KSEG1 + 0x0B000030)	/* FIR DMA Base Address Register Low (R/W) */
#define VR41XX_FIRBAHREG	preg16(KSEG1 + 0x0B000032)	/* FIR DMA Base Address Register High (R/W) */
#define VR41XX_FIRALREG		preg16(KSEG1 + 0x0B000034)	/* FIR DMA Address Register Low (R/W) */
#define VR41XX_FIRAHREG		preg16(KSEG1 + 0x0B000036)	/* FIR DMA Address Register High (R/W) */

#define VR41XX_DMARSTREG	preg16(KSEG1 + 0x0B000040)	/* DMA Reset Register (R/W) */
#define VR41XX_DMAIDLEREG	preg16(KSEG1 + 0x0B000042)	/* DMA Idle Register (R) */
#define VR41XX_DMASENREG	preg16(KSEG1 + 0x0B000044)	/* DMA Sequencer Enable Register (R/W) */
#define VR41XX_DMAMSKREG	preg16(KSEG1 + 0x0B000046)	/* DMA Mask Register (R/W) */
#define VR41XX_DMAREQREG	preg16(KSEG1 + 0x0B000048)	/* DMA Request Register (R) */
#define VR41XX_TDREG		preg16(KSEG1 + 0x0B00004A)	/* Transfer Direction Register (R/W) */

#define VR41XX_CMUCLKMSK	preg16(KSEG1 + 0x0B000060)	/* CMU Clock Mask Register (R/W) */

#define VR41XX_SYSINT1REG	preg16(KSEG1 + 0x0B000080)	/* Level 1 System interrupt register 1 (R) */
#define VR41XX_PIUINTREGro	preg16(KSEG1 + 0x0B000082)	/* Level 2 PIU interrupt register (R) */
#define VR41XX_AIUINTREG	preg16(KSEG1 + 0x0B000084)	/* Level 2 AIU interrupt register (R) */
#define VR41XX_KIUINTREG	preg16(KSEG1 + 0x0B000086)	/* Level 2 KIU interrupt register (R) */
#define VR41XX_GIUINTLREG	preg16(KSEG1 + 0x0B000088)	/* Level 2 GIU interrupt register Low (R) */
#define VR41XX_DSIUINTREG	preg16(KSEG1 + 0x0B00008A)	/* Level 2 DSIU interrupt register (R) */
#define VR41XX_MSYSINT1REG	preg16(KSEG1 + 0x0B00008C)	/* Level 1 mask system interrupt register 1 (R/W) */
#define VR41XX_MPIUINTREG	preg16(KSEG1 + 0x0B00008E)	/* Level 2 mask PIU interrupt register (R/W) */
#define VR41XX_MAIUINTREG	preg16(KSEG1 + 0x0B000090)	/* Level 2 mask AIU interrupt register (R/W) */
#define VR41XX_MKIUINTREG	preg16(KSEG1 + 0x0B000092)	/* Level 2 mask KIU interrupt register (R/W) */
#define VR41XX_MGIUINTLREG	preg16(KSEG1 + 0x0B000094)	/* Level 2 mask GIU interrupt register Low (R/W) */
#define VR41XX_MDSIUINTREG	preg16(KSEG1 + 0x0B000096)	/* Level 2 mask DSIU interrupt register (R/W) */
#define VR41XX_NMIREG		preg16(KSEG1 + 0x0B000098)	/* NMI register (R/W) */
#define VR41XX_SOFTINTREG	preg16(KSEG1 + 0x0B00009A)	/* Software interrupt register (R/W) */
#define VR41XX_SYSINT2REG	preg16(KSEG1 + 0x0B000200)	/* Level 1 System interrupt register 2 (R) */
#define VR41XX_GIUINTHREG	preg16(KSEG1 + 0x0B000202)	/* Level 2 GIU interrupt register High (R) */
#define VR41XX_FIRINTREG	preg16(KSEG1 + 0x0B000204)	/* Level 2 FIR interrupt register (R) */
#define VR41XX_MSYSINT2REG	preg16(KSEG1 + 0x0B000206)	/* Level 1 mask system interrupt register 2 (R/W) */
#define VR41XX_MGIUINTHREG	preg16(KSEG1 + 0x0B000208)	/* Level 2 mask GIU interrupt register High (R/W) */
#define VR41XX_MFIRINTREG	preg16(KSEG1 + 0x0B00020A)	/* Level 2 mask FIR interrupt register (R/W) */

#define VR41XX_PMUINTREG	preg16(KSEG1 + 0x0B0000A0)	/* PMU Interrupt/Status Register (R/W) */
#define VR41XX_PMUCNTREG	preg16(KSEG1 + 0x0B0000A2)	/* PMU Control Register (R/W) */
#define VR41XX_PMUINT2REG	preg16(KSEG1 + 0x0B0000A4)	/* PMU Interrupt/Status 2 Register (R/W) */
#define VR41XX_PMUCNT2REG	preg16(KSEG1 + 0x0B0000A6)	/* PMU Control 2 Resister (R/W) */
#define VR41XX_PMUWAITREG	preg16(KSEG1 + 0x0B0000A8)	/* PMU Wait Counter Register (R/W) */

#define VR41XX_ETIMELREG	preg16(KSEG1 + 0x0B0000C0)	/* Elapsed Time L Register (R/W) */
#define VR41XX_ETIMEMREG	preg16(KSEG1 + 0x0B0000C2)	/* Elapsed Time M Register (R/W) */
#define VR41XX_ETIMEHREG	preg16(KSEG1 + 0x0B0000C4)	/* Elapsed Time H Register (R/W) */
#define VR41XX_ECMPLREG		preg16(KSEG1 + 0x0B0000C8)	/* Elapsed Compare L Register (R/W) */
#define VR41XX_ECMPMREG		preg16(KSEG1 + 0x0B0000CA)	/* Elapsed Compare M Register (R/W) */
#define VR41XX_ECMPHREG		preg16(KSEG1 + 0x0B0000CC)	/* Elapsed Compare H Register (R/W) */
#define VR41XX_RTCL1LREG	preg16(KSEG1 + 0x0B0000D0)	/* RTC Long 1 L Register (R/W) */
#define VR41XX_RTCL1HREG	preg16(KSEG1 + 0x0B0000D2)	/* RTC Long 1 H Register (R/W) */
#define VR41XX_RTCL1CNTLREG	preg16(KSEG1 + 0x0B0000D4)	/* RTC Long 1 Count L Register (R) */
#define VR41XX_RTCL1CNTHREG	preg16(KSEG1 + 0x0B0000D6)	/* RTC Long 1 Count H Register (R) */
#define VR41XX_RTCL2LREG	preg16(KSEG1 + 0x0B0000D8)	/* RTC Long 2 L Register (R/W) */
#define VR41XX_RTCL2HREG	preg16(KSEG1 + 0x0B0000DA)	/* RTC Long 2 H Register (R/W) */
#define VR41XX_RTCL2CNTLREG	preg16(KSEG1 + 0x0B0000DC)	/* RTC Long 2 Count L Register (R) */
#define VR41XX_RTCL2CNTHREG	preg16(KSEG1 + 0x0B0000DE)	/* RTC Long 2 Count H Register (R) */
#define VR41XX_TCLKLREG		preg16(KSEG1 + 0x0B0001C0)	/* TCLK L Register (R/W) */
#define VR41XX_TCLKHREG		preg16(KSEG1 + 0x0B0001C2)	/* TCLK H Register (R/W) */
#define VR41XX_TCLKCNTLREG	preg16(KSEG1 + 0x0B0001C4)	/* TCLK Count L Register (R) */
#define VR41XX_TCLKCNTHREG	preg16(KSEG1 + 0x0B0001C6)	/* TCLK Count H Register (R) */
#define VR41XX_RTCINTREG	preg16(KSEG1 + 0x0B0001DE)	/* RTC Interrupt Register (R/W) */

#define VR41XX_DSUCNTREG	preg16(KSEG1 + 0x0B0000E0)	/* DSU Control Register (R/W) */
#define VR41XX_DSUSETREG	preg16(KSEG1 + 0x0B0000E2)	/* DSU Dead Time Set Register (R/W) */
#define VR41XX_DSUCLRREG	preg16(KSEG1 + 0x0B0000E4)	/* DSU Clear Register (W) */
#define VR41XX_DSUTIMREG	preg16(KSEG1 + 0x0B0000E6)	/* DSU Elapsed Time Register (R/W) */

#define VR41XX_GIUIOSELL	preg16(KSEG1 + 0x0B000100)	/* GPIO Input/Output Select Register L (R/W) */
#define VR41XX_GIUIOSELH	preg16(KSEG1 + 0x0B000102)	/* GPIO Input/Output Select Register H (R/W) */
#define VR41XX_GIUPIODL		preg16(KSEG1 + 0x0B000104)	/* GPIO Port Input/Output Data Register L (R/W) */
#define VR41XX_GIUPIODH		preg16(KSEG1 + 0x0B000106)	/* GPIO Port Input/Output Data Register H (R/W) */
#define VR41XX_GIUINTSTATL	preg16(KSEG1 + 0x0B000108)	/* GPIO Interrupt Status Register L (R/W) */
#define VR41XX_GIUINTSTATH	preg16(KSEG1 + 0x0B00010A)	/* GPIO Interrupt Status Register H (R/W) */
#define VR41XX_GIUINTENL	preg16(KSEG1 + 0x0B00010C)	/* GPIO Interrupt Enable Register L (R/W) */
#define VR41XX_GIUINTENH	preg16(KSEG1 + 0x0B00010E)	/* GPIO Interrupt Enable Register H (R/W) */
#define VR41XX_GIUINTTYPL	preg16(KSEG1 + 0x0B000110)	/* GPIO Interrupt Type (Edge or Level) Select Register (R/W) */
#define VR41XX_GIUINTTYPH	preg16(KSEG1 + 0x0B000112)	/* GPIO Interrupt Type (Edge or Level) Select Register (R/W) */
#define VR41XX_GIUINTALSELL	preg16(KSEG1 + 0x0B000114)	/* GPIO Interrupt Active Level Select Register L (R/W) */
#define VR41XX_GIUINTALSELH	preg16(KSEG1 + 0x0B000116)	/* GPIO Interrupt Active Level Select Register H (R/W) */
#define VR41XX_GIUINTHTSELL	preg16(KSEG1 + 0x0B000118)	/* GPIO Interrupt Hold/Through Select Register L (R/W) */
#define VR41XX_GIUINTHTSELH	preg16(KSEG1 + 0x0B00011A)	/* GPIO Interrupt Hold/Through Select Register H (R/W) */

#define VR41XX_GIUPODATL	preg16(KSEG1 + 0x0B00011C)	/* GPIO Port Output Data Register L (R/W) */
#define VR41XX_GIUPODATL_GPIO47  0x8000
#define VR41XX_GIUPODATL_GPIO46  0x4000
#define VR41XX_GIUPODATL_GPIO45  0x2000
#define VR41XX_GIUPODATL_GPIO44  0x1000
#define VR41XX_GIUPODATL_GPIO43  0x0800
#define VR41XX_GIUPODATL_GPIO42  0x0400
#define VR41XX_GIUPODATL_GPIO41  0x0200
#define VR41XX_GIUPODATL_GPIO40  0x0100
#define VR41XX_GIUPODATL_GPIO39  0x0080
#define VR41XX_GIUPODATL_GPIO38  0x0040
#define VR41XX_GIUPODATL_GPIO37  0x0020
#define VR41XX_GIUPODATL_GPIO36  0x0010
#define VR41XX_GIUPODATL_GPIO35  0x0008
#define VR41XX_GIUPODATL_GPIO34  0x0004
#define VR41XX_GIUPODATL_GPIO33  0x0002
#define VR41XX_GIUPODATL_GPIO32  0x0001

#define VR41XX_GIUPODATH	preg16(KSEG1 + 0x0B00011E)	/* GPIO Port Output Data Register H (R/W) */
#define VR41XX_GIUUSEUPDN	preg16(KSEG1 + 0x0B0002E0)	/* GPIO Pullup/Down User Register (R/W) */
#define VR41XX_GIUTERMUPDN	preg16(KSEG1 + 0x0B0002E2)	/* GPIO Terminal Pullup/Down Register (R/W) */

#define VR41XX_PIUCNTREG	preg16(KSEG1 + 0x0B000122)	/* PIU Control register (R/W) */
#define VR41XX_PIUCNTREG_PIUSEQEN	0x0040

#define VR41XX_PIUINTREG	preg16(KSEG1 + 0x0B000124)	/* PIU Interrupt cause register (R/W) */
#define VR41XX_PIUINTREG_OVP		0x8000
#define VR41XX_PIUINTREG_PADCMD		0x0040
#define VR41XX_PIUINTREG_PADADP		0x0020
#define VR41XX_PIUINTREG_PADPAGE1	0x0010
#define VR41XX_PIUINTREG_PADPAGE0	0x0008
#define VR41XX_PIUINTREG_PADDLOST	0x0004
#define VR41XX_PIUINTREG_PENCHG		0x0001

#define VR41XX_PIUSIVLREG	preg16(KSEG1 + 0x0B000126)	/* PIU Data sampling interval register (R/W) */
#define VR41XX_PIUSTBLREG	preg16(KSEG1 + 0x0B000128)	/* PIU A/D converter start delay register (R/W) */
#define VR41XX_PIUCMDREG	preg16(KSEG1 + 0x0B00012A)	/* PIU A/D command register (R/W) */
#define VR41XX_PIUASCNREG	preg16(KSEG1 + 0x0B000130)	/* PIU A/D port scan register (R/W) */
#define VR41XX_PIUAMSKREG	preg16(KSEG1 + 0x0B000132)	/* PIU A/D scan mask register (R/W) */
#define VR41XX_PIUCIVLREG	preg16(KSEG1 + 0x0B00013E)	/* PIU Check interval register (R) */
#define VR41XX_PIUPB00REG	preg16(KSEG1 + 0x0B0002A0)	/* PIU Page 0 Buffer 0 register (R/W) */
#define VR41XX_PIUPB01REG	preg16(KSEG1 + 0x0B0002A2)	/* PIU Page 0 Buffer 1 register (R/W) */
#define VR41XX_PIUPB02REG	preg16(KSEG1 + 0x0B0002A4)	/* PIU Page 0 Buffer 2 register (R/W) */
#define VR41XX_PIUPB03REG	preg16(KSEG1 + 0x0B0002A6)	/* PIU Page 0 Buffer 3 register (R/W) */
#define VR41XX_PIUPB10REG	preg16(KSEG1 + 0x0B0002A8)	/* PIU Page 1 Buffer 0 register (R/W) */
#define VR41XX_PIUPB11REG	preg16(KSEG1 + 0x0B0002AA)	/* PIU Page 1 Buffer 1 register (R/W) */
#define VR41XX_PIUPB12REG	preg16(KSEG1 + 0x0B0002AC)	/* PIU Page 1 Buffer 2 register (R/W) */
#define VR41XX_PIUPB13REG	preg16(KSEG1 + 0x0B0002AE)	/* PIU Page 1 Buffer 3 register (R/W) */
#define VR41XX_PIUAB0REG	preg16(KSEG1 + 0x0B0002B0)	/* PIU A/D scan Buffer 0 register (R/W) */
#define VR41XX_PIUAB1REG	preg16(KSEG1 + 0x0B0002B2)	/* PIU A/D scan Buffer 1 register (R/W) */
#define VR41XX_PIUAB2REG	preg16(KSEG1 + 0x0B0002B4)	/* PIU A/D scan Buffer 2 register (R/W) */
#define VR41XX_PIUAB3REG	preg16(KSEG1 + 0x0B0002B6)	/* PIU A/D scan Buffer 3 register (R/W) */
#define VR41XX_PIUPB04REG	preg16(KSEG1 + 0x0B0002BC)	/* PIU Page 0 Buffer 4 register (R/W) */
#define VR41XX_PIUPB14REG	preg16(KSEG1 + 0x0B0002BE)	/* PIU Page 1 Buffer 4 register (R/W) */

#define VR41XX_MDMADATREG	preg16(KSEG1 + 0x0B000160)	/* Mike DMA Data Register (R/W) */
#define VR41XX_SDMADATREG	preg16(KSEG1 + 0x0B000162)	/* Speaker DMA Data Register (R/W) */
#define VR41XX_SODATREG		preg16(KSEG1 + 0x0B000166)	/* Speaker Output Data Register (R/W) */
#define VR41XX_SCNTREG		preg16(KSEG1 + 0x0B000168)	/* Speaker Output Control Register (R/W) */
#define VR41XX_SCNVRREG		preg16(KSEG1 + 0x0B00016A)	/* Speaker Conversion Rate Register (R/W) */
#define VR41XX_MIDATREG		preg16(KSEG1 + 0x0B000170)	/* Mike Input Data Register (R/W) */
#define VR41XX_MCNTREG		preg16(KSEG1 + 0x0B000172)	/* Mike Input Control Register (R/W) */
#define VR41XX_MCNVRREG		preg16(KSEG1 + 0x0B000174)	/* Mike Conversion Rate Register (R/W) */
#define VR41XX_DVALIDREG	preg16(KSEG1 + 0x0B000178)	/* Data Valid Register (R/W) */
#define VR41XX_SEQREG		preg16(KSEG1 + 0x0B00017A)	/* Sequential Register (R/W) */
#define VR41XX_INTREG		preg16(KSEG1 + 0x0B00017C)	/* Interrupt Register (R/W) */

#define VR41XX_KIUDAT0		preg16(KSEG1 + 0x0B000180)	/* KIU Data0 Register (R/W) */
#define VR41XX_KIUDAT1		preg16(KSEG1 + 0x0B000182)	/* KIU Data1 Register (R/W) */
#define VR41XX_KIUDAT2		preg16(KSEG1 + 0x0B000184)	/* KIU Data2 Register (R/W) */
#define VR41XX_KIUDAT3		preg16(KSEG1 + 0x0B000186)	/* KIU Data3 Register (R/W) */
#define VR41XX_KIUDAT4		preg16(KSEG1 + 0x0B000188)	/* KIU Data4 Register (R/W) */
#define VR41XX_KIUDAT5		preg16(KSEG1 + 0x0B00018A)	/* KIU Data5 Register (R/W) */
#define VR41XX_KIUSCANREP	preg16(KSEG1 + 0x0B000190)	/* KIU Scan/Repeat Register (R/W) */
#define VR41XX_KEYEN     0x8000
#define VR41XX_SCANSTP   0x0008
#define VR41XX_SCANSTART 0x0004
#define VR41XX_ATSTP     0x0002
#define VR41XX_ATSCAN    0x0001
#define VR41XX_KIUSCANS		preg16(KSEG1 + 0x0B000192)	/* KIU Scan Status Register (R) */
#define VR41XX_KIUWKS		preg16(KSEG1 + 0x0B000194)	/* KIU Wait Keyscan Stable Register (R/W) */
#define VR41XX_KIUWKI		preg16(KSEG1 + 0x0B000196)	/* KIU Wait Keyscan Interval Register (R/W) */
#define VR41XX_KIUINT		preg16(KSEG1 + 0x0B000198)	/* KIU Interrupt Register (R/W) */
#define VR41XX_KDATLOST  0x0004
#define VR41XX_KDATRDY   0x0002
#define VR41XX_SCANINT   0x0001
#define VR41XX_KIURST		preg16(KSEG1 + 0x0B00019A)	/* KIU Reset Register (W) */
#define VR41XX_KIUGPEN		preg16(KSEG1 + 0x0B00019C)	/* KIU General Purpose Output Enable (R/W) */
#define VR41XX_SCANLINE		preg16(KSEG1 + 0x0B00019E)	/* KIU Scan Line Register (R/W) */

#define VR41XX_PORTREG		preg16(KSEG1 + 0x0B0001A0)	/* Port Change Register (R/W) */
#define VR41XX_MODEMREG		preg16(KSEG1 + 0x0B0001A2)	/* Modem Control Register (R) */
#define VR41XX_ASIM00REG	preg16(KSEG1 + 0x0B0001A4)	/* Asynchronous Mode 0 Register (R/W) */
#define VR41XX_ASIM01REG	preg16(KSEG1 + 0x0B0001A6)	/* Asynchronous Mode 1 Register (R/W) */
#define VR41XX_RXB0RREG		preg16(KSEG1 + 0x0B0001A8)	/* Receive Buffer Register (Extended) (R) */
#define VR41XX_RXB0LREG		preg16(KSEG1 + 0x0B0001AA)	/* Receive Buffer Register (R) */
#define VR41XX_TXS0RREG		preg16(KSEG1 + 0x0B0001AC)	/* Transmit Data Register (Extended) (R/W) */
#define VR41XX_TXS0LREG		preg16(KSEG1 + 0x0B0001AE)	/* Transmit Data Register (R/W) */
#define VR41XX_ASIS0REG		preg16(KSEG1 + 0x0B0001B0)	/* Status Register (R) */
#define VR41XX_INTR0REG		preg16(KSEG1 + 0x0B0001B2)	/* Debug SIU Interrupt Register (R/W) */
#define VR41XX_BPRM0REG		preg16(KSEG1 + 0x0B0001B6)	/* Baud rate Generator Prescaler Mode Register (R/W) */
#define VR41XX_DSIURESETREG	preg16(KSEG1 + 0x0B0001B8)	/* Debug SIU Reset Register (R/W) */

#define VR41XX_LEDHTSREG	preg16(KSEG1 + 0x0B000240)	/* LED H Time Set register (R/W) */
#define VR41XX_LEDLTSREG	preg16(KSEG1 + 0x0B000242)	/* LED L Time Set register (R/W) */
#define VR41XX_LEDCNTREG	preg16(KSEG1 + 0x0B000248)	/* LED Control register (R/W) */
#define VR41XX_LEDASTCREG	preg16(KSEG1 + 0x0B00024A)	/* LED Auto Stop Time Count register (R/W) */
#define VR41XX_LEDINTREG	preg16(KSEG1 + 0x0B00024C)	/* LED Interrupt register (R/W) */

#define VR41XX_SIURB		preg8(KSEG1 + 0x0C000000)	/* Receiver Buffer Register (Read) DLAB = 0 (R) */
#define VR41XX_SIUTH		preg8(KSEG1 + 0x0C000000)	/* Transmitter Holding Register (Write) DLAB = 0 (W) */
#define VR41XX_SIUDLL		preg8(KSEG1 + 0x0C000000)	/* Divisor Latch (Least Significant Byte) DLAB = 1 (R/W) */
#define VR41XX_SIUIE		preg8(KSEG1 + 0x0C000001)	/* Interrupt Enable DLAB = 0 (R/W) */
#define VR41XX_SIUDLM		preg8(KSEG1 + 0x0C000001)	/* Divisor Latch (Most Significant Byte) DLAB = 1 (R/W) */
#define VR41XX_SIUIID		preg8(KSEG1 + 0x0C000002)	/* Interrupt Identification Register (Read) (R) */
#define VR41XX_SIUFC		preg8(KSEG1 + 0x0C000002)	/* FIFO Control Register (Write) (W) */
#define VR41XX_SIULC		preg8(KSEG1 + 0x0C000003)	/* Line Control Register (R/W) */
#define VR41XX_SIUMC		preg8(KSEG1 + 0x0C000004)	/* MODEM Control Register (R/W) */
#define VR41XX_SIULS		preg8(KSEG1 + 0x0C000005)	/* Line Status Register (R/W) */
#define VR41XX_SIUMS		preg8(KSEG1 + 0x0C000006)	/* MODEM Status Register (R/W) */
#define VR41XX_SIUSC		preg8(KSEG1 + 0x0C000007)	/* Scratch Register (R/W) */
#define VR41XX_SIUIRSEL		preg8(KSEG1 + 0x0C000008)	/* SIU/FIR IrDA Selector (R/W) */
#define VR41XX_SIURESET		preg8(KSEG1 + 0x0C000009)	/* SIU Reset Register (R/W) */
#define VR41XX_SIUCSEL		preg8(KSEG1 + 0x0C00000A)	/* SIU Echo-Back Control Register (R/W) */

	/* Not sure if some of these are right type, some may be 16 bit regs: */
#define VR41XX_HSPINIT		preg8(KSEG1 + 0x0C000020)	/* HSP Initialize Register (R/W) */
#define VR41XX_HSPDATAL		preg8(KSEG1 + 0x0C000022)	/* HSP Data Register L (R/W) */
#define VR41XX_HSPDATAH		preg8(KSEG1 + 0x0C000023)	/* HSP Data Register H (R/W) */
#define VR41XX_HSPINDEX		preg8(KSEG1 + 0x0C000024)	/* HSP Index Register (W) */
#define VR41XX_HSPID		preg8(KSEG1 + 0x0C000028)	/* HSP ID Register (R) */
#define VR41XX_HSPPCS		preg8(KSEG1 + 0x0C000029)	/* HSP I/O Address Program Confirmation Register (R) */
#define VR41XX_HSPPCTEL		preg8(KSEG1 + 0x0C000029)	/* HSP Signature Checking Port (W) */

#define VR41XX_FRSTR		preg16(KSEG1 + 0x0C000040)	/* FIR Reset register (R/W) */
#define VR41XX_DPINTR		preg16(KSEG1 + 0x0C000042)	/* DMA Page Interrupt register (R/W) */
#define VR41XX_DPCNTR		preg16(KSEG1 + 0x0C000044)	/* DMA Control register (R/W) */
#define VR41XX_TDR		preg16(KSEG1 + 0x0C000050)	/* Transmit Data register (W) */
#define VR41XX_RDR		preg16(KSEG1 + 0x0C000052)	/* Receive Data register (R) */
#define VR41XX_IMR		preg16(KSEG1 + 0x0C000054)	/* Interrupt Mask register (R/W) */
#define VR41XX_FSR		preg16(KSEG1 + 0x0C000056)	/* FIFO Setup register (R/W) */
#define VR41XX_IRSR1		preg16(KSEG1 + 0x0C000058)	/* Infrared Setup register 1 (R/W) */
#define VR41XX_CRCSR		preg16(KSEG1 + 0x0C00005C)	/* CRC Setup register (R/W) */
#define VR41XX_FIRCR		preg16(KSEG1 + 0x0C00005E)	/* FIR Control register (R/W) */
#define VR41XX_MIRCR		preg16(KSEG1 + 0x0C000060)	/* MIR Control register (R/W) */
#define VR41XX_DMACR		preg16(KSEG1 + 0x0C000062)	/* DMA Control register (R/W) */
#define VR41XX_DMAER		preg16(KSEG1 + 0x0C000064)	/* DMA Enable register (R/W) */
#define VR41XX_TXIR		preg16(KSEG1 + 0x0C000066)	/* Transmit Indication register (R) */
#define VR41XX_RXIR		preg16(KSEG1 + 0x0C000068)	/* Receive Indication register (R) */
#define VR41XX_IFR		preg16(KSEG1 + 0x0C00006A)	/* Interrupt Flag register (R) */
#define VR41XX_RXSTS		preg16(KSEG1 + 0x0C00006C)	/* Receive Status (R) */
#define VR41XX_TXFL		preg16(KSEG1 + 0x0C00006E)	/* Transmit Frame Length (R/W) */
#define VR41XX_MRXF		preg16(KSEG1 + 0x0C000070)	/* Maximum Receive Frame Length (R/W) */
#define VR41XX_RXFL		preg16(KSEG1 + 0x0C000074)	/* Receive Frame Length (R) */

// physical address spaces
#define VR41XX_LCD             0x0a000000
#define VR41XX_INTERNAL_IO_2   0x0b000000
#define VR41XX_INTERNAL_IO_1   0x0c000000
#define VR41XX_ISA_MEM         0x10000000
#define VR41XX_ISA_IO          0x14000000
#define VR41XX_ROM             0x18000000

// This is the base address for IO port decoding to which the 16 bit IO port address
// is added.  Defining it to 0 will usually cause a kernel oops any time port IO is
// attempted, which can be handy for turning up parts of the kernel that make
// incorrect architecture assumptions (by assuming that everything acts like a PC),
// but we need it correctly defined to use the PCMCIA/CF controller:
#define VR41XX_PORT_BASE	(KSEG1 + VR41XX_ISA_IO)

#endif /* __ASM_MIPS_VR41XX_H */
