/*
 * Simple framebuffer defines
 */

#ifndef _VIDEO_SFB_H
#define _VIDEO_SFB_H

#include <linux/config.h>
#include <asm/addrspace.h>

/*
 * VIDEORAM_BASE     Address of start of video ram
 * VIDEORAM_SIZE     Normally x_res * y_res * (bpp/8) 
 * FB_X_RES          Visible horizontal resolution in pixels
 * FB_X_VIRTUAL_RES  Horizontal resolution of framebuffer memory in pixels
 *                   (default is FB_X_RES)
 * FB_Y_RES          Visible vertical resolution in pixels
 * FB_Y_VIRTUAL_RES  Vertical resolution of framebuffer memory in pixels
 *                   (default FB_Y_RES)
 * FB_BPP            Bits per pixel
 * FB_IS_GREY        Greyscale
 * FB_INVERSE        1 means pallette 0 is white instead of black
 */

#ifdef CONFIG_CASIO_E10
#define VIDEORAM_BASE     (KSEG1 + 0x0a000000)
#define VIDEORAM_SIZE     320 * (1024 * 2 / 8)
#define FB_X_RES          240
#define FB_X_VIRTUAL_RES  1024
#define FB_Y_RES          320
#define FB_BPP            2
#define FB_IS_GREY        1
#define FB_INVERSE        0
#endif

#ifdef CONFIG_CASIO_E15
#define VIDEORAM_BASE     (KSEG1 + 0x0a000000)
#define VIDEORAM_SIZE     (256 * 1024) /* ??? */
#define FB_X_RES          240
#define FB_X_VIRTUAL_RES  512
#define FB_Y_RES          320
#define FB_BPP            4
#define FB_IS_GREY        1
#define FB_INVERSE        1
#endif

#ifdef CONFIG_CASIO_E105
#define VIDEORAM_BASE     (KSEG1 + 0x0a200000)
#define VIDEORAM_SIZE     (320 * 0x200)
#define FB_X_RES          240
#define FB_X_VIRTUAL_RES  256
#define FB_Y_RES          320
#define FB_BPP            16
#define FB_IS_GREY        0
#define FB_INVERSE        0
#endif

#ifdef CONFIG_EVEREX_FREESTYLE
#define VIDEORAM_BASE  (KSEG1 + 0x0a000000)
#define VIDEORAM_SIZE  (256 * 1024)
#define FB_X_RES       320
#define FB_Y_RES       240
#define FB_BPP         2
#define FB_IS_GREY     1
#define FB_INVERSE     1
#endif

#ifdef CONFIG_NEC_MOBILEPRO_700
#define VIDEORAM_BASE  (KSEG1 + 0x0a000000)
#define VIDEORAM_SIZE  (64 * 1024)
#define FB_X_VIRTUAL_RES  1024
#define FB_X_RES       640
#define FB_Y_RES       240
#define FB_BPP         2
#define FB_IS_GREY     1
#define FB_INVERSE     0
#endif

#endif /* _VIDEO_SFB_H */
